/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI
 * @author     Raphael ( raphael dot grimm at student dot kit dot edu )
 * @date       2017
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include "SensorValueBase.h"

#include <ArmarXCore/core/util/algorithm.h>
#include <RobotAPI/interface/units/KinematicUnitInterface.h>
#include <RobotAPI/interface/units/KinematicUnitInterfaceStdOverloads.h>

namespace armarx
{
    class SensorValue1DoFActuatorPosition : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float position = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorPosition> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorPosition> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorPosition::position, "position");
            return svi;
        }
    };
    class SensorValue1DoFActuatorVelocity : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float velocity = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorVelocity> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorVelocity> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorVelocity::velocity, "velocity");
            return svi;
        }
    };
    class SensorValue1DoFActuatorFilteredVelocity : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float filteredvelocity = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorFilteredVelocity> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorFilteredVelocity> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorFilteredVelocity::filteredvelocity, "filteredvelocity");
            return svi;
        }
    };
    class SensorValue1DoFActuatorAcceleration : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float acceleration = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorAcceleration> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorAcceleration> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorAcceleration::acceleration, "acceleration");
            return svi;
        }
    };
    class SensorValue1DoFActuatorTorque : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float torque = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorTorque> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorTorque> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorTorque::torque, "torque");
            return svi;
        }
    };
    class SensorValue1DoFGravityTorque : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float gravityTorque = 0.0f;
        static SensorValueInfo<SensorValue1DoFGravityTorque> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFGravityTorque> svi;
            svi.addMemberVariable(&SensorValue1DoFGravityTorque::gravityTorque, "gravityTorque");
            return svi;
        }
    };


    class SensorValue1DoFActuator :
        virtual public SensorValue1DoFActuatorPosition,
        virtual public SensorValue1DoFActuatorVelocity,
        virtual public SensorValue1DoFActuatorAcceleration,
        virtual public SensorValue1DoFActuatorTorque,
        virtual public SensorValue1DoFGravityTorque
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        static SensorValueInfo<SensorValue1DoFActuator> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuator> svi;
            svi.addBaseClass<SensorValue1DoFActuatorPosition>();
            svi.addBaseClass<SensorValue1DoFActuatorVelocity>();
            svi.addBaseClass<SensorValue1DoFActuatorAcceleration>();
            svi.addBaseClass<SensorValue1DoFActuatorTorque>();
            //svi.addBaseClass<SensorValue1DoFGravityTorque>();
            return svi;
        }
    };


    class SensorValue1DoFActuatorCurrent : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float motorCurrent = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorCurrent> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorCurrent> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorCurrent::motorCurrent, "motorCurrent");
            return svi;
        }
    };
    class SensorValue1DoFActuatorMotorTemperature : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        float motorTemperature = 0.0f;
        static SensorValueInfo<SensorValue1DoFActuatorMotorTemperature> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorMotorTemperature> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorMotorTemperature::motorTemperature, "motorTemperature");
            return svi;
        }
    };

    class SensorValue1DoFActuatorStatus : virtual public SensorValueBase
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        armarx::JointStatus status;
        static SensorValueInfo<SensorValue1DoFActuatorStatus> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFActuatorStatus> svi;
            svi.addMemberVariable(&SensorValue1DoFActuatorStatus::status, "JointStatus")
            .setFieldNames({"JointStatusError", "JointStatusOperation", "JointStatusEnabled", "JointStatusEmergencyStop"});
            return svi;
        }
    };

    class SensorValue1DoFRealActuator :
        virtual public SensorValue1DoFActuator,
        virtual public SensorValue1DoFActuatorCurrent,
        virtual public SensorValue1DoFActuatorMotorTemperature
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        static SensorValueInfo<SensorValue1DoFRealActuator> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFRealActuator> svi;
            svi.addBaseClass<SensorValue1DoFActuator>();
            svi.addBaseClass<SensorValue1DoFActuatorCurrent>();
            svi.addBaseClass<SensorValue1DoFActuatorMotorTemperature>();
            return svi;
        }
    };

    class SensorValue1DoFRealActuatorWithStatus :
        virtual public SensorValue1DoFRealActuator,
        virtual public SensorValue1DoFActuatorStatus
    {
    public:
        DETAIL_SensorValueBase_DEFAULT_METHOD_IMPLEMENTATION
        static SensorValueInfo<SensorValue1DoFRealActuatorWithStatus> GetClassMemberInfo()
        {
            SensorValueInfo<SensorValue1DoFRealActuatorWithStatus> svi;
            svi.addBaseClass<SensorValue1DoFRealActuator>();
            svi.addBaseClass<SensorValue1DoFActuatorStatus>();
            return svi;
        }
    };
}
